package org.elvior.thermometer;

import org.elvior.ttcn.tritci.TriFactory;
import org.elvior.ttcn.tritci.TriProvider;
import org.elvior.ttcn.tritci.TriTciChannel;
import org.elvior.ttcn.tritci.ChannelEventHandler;
import org.etsi.ttcn.tri.*;

public class Comm implements TriCommunicationSA, TriPlatformPA, ChannelEventHandler {

	private TriComponentId m_componentId;
    private TriPortId m_tsiPortId;
    private TriStatus m_triOK;
    private Analyzer m_analyzer;
    
    private static final String SENT = "sent";
    private static final String RECEIVED = "received";
    private final int MESSAGE_HEADER_LENGTH = 8;
    
	public Comm() {
		// Register this object as the instance implementing the TRI-SA interface
		TriProvider.getInstance().setTriCommunicationSA(this);
		// Register this object as the instance implementing the TRI-PA interface
		TriProvider.getInstance().setTriPlatformPA(this);		
		TriTciChannel channel = TriTciChannel.getInstance();
		// Set application name that will be displayed in the TestCast log
		channel.setApplicationName("Thermometer");
		// Register this object as the receiver of events generated by TestCast framework
		channel.setEventHandler(this);
		TriFactory factory = TriProvider.getInstance().getTriFactory(); 
		m_triOK = factory.createOkStatus();
		
		m_analyzer = new Analyzer();
	}
	
	public void run() {
		// Attempt to connect to TestCast
		if (TriTciChannel.getInstance().open())
			try {
				synchronized(this) {
					wait();
				}
			} catch (InterruptedException e) {
		}		
	}
	
	public void enqueueMessage(byte[] ayData) {
        printMessageInfo(ayData, SENT);
        TriMessage msgToSend = TriProvider.getInstance().getTriFactory().createMessage();
        msgToSend.setEncodedMessage(ayData);
        TriProvider.getInstance().getTriCommunicationTE().triEnqueueMsg(m_tsiPortId, null, 
            m_componentId, msgToSend);
    }
	
	//Thermometer has received a message
    private void processReceivedMessage(byte[] ayMsg) {
        printMessageInfo(ayMsg, RECEIVED);

        ThermometerMsg msg;
        if (ayMsg.length < MESSAGE_HEADER_LENGTH) {
            //Error: message size is smaller than the header size should be
            msg = m_analyzer.createErrorMsg(ErrorReply.ERR_HDR_TOO_SHORT);
        }
        else {
            int nMsgCode = BitConverter.toInt(ayMsg, 0);
            int nPayloadLen = BitConverter.toInt(ayMsg, 4);

            byte[] ayPayload = new byte[nPayloadLen];
            System.arraycopy(ayMsg, MESSAGE_HEADER_LENGTH, ayPayload, 0, nPayloadLen);
            msg = m_analyzer.analyzeMsg(nMsgCode, ayPayload);
        }
        enqueueMessage(msg.encode());
    }
       
    private synchronized void log(String sMsg){
    	System.out.println(sMsg);
    }
    
	private void printMessageInfo(byte[] ayMsg, String sActionName) {
		int nMsgCode = 0;
        if (ayMsg.length >= 4)
            nMsgCode = BitConverter.toInt(ayMsg, 0);
        String sMessageName = "";
        if (nMsgCode == TempRequest.CODE)
            sMessageName = "TempRequest";
        else if (nMsgCode == TempReply.CODE)
            sMessageName = "TempReply";
        else if (nMsgCode == HumRequest.CODE)
            sMessageName = "HumRequest";
        else if (nMsgCode == HumReply.CODE)
            sMessageName = "HumReply";
        else if (nMsgCode == ErrorReply.CODE)
            sMessageName = "ErrorReply";
        else
            sMessageName = "unknown";
        log(sActionName + " " + sMessageName + " (message code " + nMsgCode + ")");
	}

	public TriStatus triCall(TriComponentId componentId, TriPortId tsiPortId,
			TriAddress sutAddress, TriSignatureId signatureId,
			TriParameterList parameterList) {
		return m_triOK;
	}

	public TriStatus triCallBC(TriComponentId componentId, TriPortId tsiPortId,
			TriSignatureId signatureId, TriParameterList parameterList) {
		return m_triOK;
	}

	public TriStatus triCallMC(TriComponentId componentId, TriPortId tsiPortId,
			TriAddressList sutAddresses, TriSignatureId signatureId,
			TriParameterList parameterList) {
		return m_triOK;
	}

	public TriStatus triEndTestCase() {
		return m_triOK;
	}

	public TriStatus triExecuteTestCase(TriTestCaseId testCaseId,
			TriPortIdList tsiPorts) {
		return m_triOK;
	}

	public TriStatus triMap(TriPortId compPortId, TriPortId tsiPortId) {
		//saving values that are later needed in triSend
        //see step #5 under chapter "Step 1: System and Platform Adapter" from the doc
        m_componentId = (TriComponentId)compPortId.getComponent();
        m_tsiPortId = (TriPortId)tsiPortId;
        return m_triOK;
	}

	public TriStatus triRaise(TriComponentId componentId, TriPortId tsitPortId,
			TriAddress sutAddress, TriSignatureId signatureId, TriException exc) {
		return m_triOK;
	}

	public TriStatus triRaiseBC(TriComponentId componentId,
			TriPortId tsitPortId, TriSignatureId signatureId, TriException exc) {
		return m_triOK;
	}

	public TriStatus triRaiseMC(TriComponentId componentId,
			TriPortId tsitPortId, TriAddressList sutAddresses,
			TriSignatureId signatureId, TriException exc) {
		return m_triOK;
	}

	public TriStatus triReply(TriComponentId componentId, TriPortId tsiPortId,
			TriAddress sutAddress, TriSignatureId signatureId,
			TriParameterList parameterList, TriParameter returnValue) {
		return m_triOK;
	}

	public TriStatus triReplyBC(TriComponentId componentId,
			TriPortId tsiPortId, TriSignatureId signatureId,
			TriParameterList parameterList, TriParameter returnValue) {
		return m_triOK;
	}

	public TriStatus triReplyMC(TriComponentId componentId,
			TriPortId tsiPortId, TriAddressList sutAddresses,
			TriSignatureId signatureId, TriParameterList parameterList,
			TriParameter returnValue) {
		return m_triOK;
	}

	public TriStatus triSAReset() {
		return m_triOK;
	}

	public TriStatus triSend(TriComponentId componentId, TriPortId tsiPortId,
			TriAddress address, TriMessage sendMessage) {
		processReceivedMessage(sendMessage.getEncodedMessage());
		return m_triOK;
	}

	public TriStatus triSendBC(TriComponentId componentId, TriPortId tsiPortId,
			TriMessage sendMessage) {
		processReceivedMessage(sendMessage.getEncodedMessage());
		return m_triOK;
	}

	public TriStatus triSendMC(TriComponentId componentId, TriPortId tsiPortId,
			TriAddressList addresses, TriMessage sendMessage) {
		processReceivedMessage(sendMessage.getEncodedMessage());
		return m_triOK;
	}

	public TriStatus triSutActionInformal(String description) {
		return m_triOK;
	}

	public TriStatus triUnmap(TriPortId compPortId, TriPortId tsiPortId) {
		return m_triOK;
	}

	public TriStatus triExternalFunction(TriFunctionId functionId,
			TriParameterList parameterList, TriParameter returnValue) {
			String functionName = functionId.getFunctionName();//qualified name here
        int nIdx = functionName.indexOf('.');
        if(nIdx != -1) functionName = functionName.substring(nIdx + 1);//remove module name

     if(functionName.equals("add")){
      int o1 = BitConverter.toInt(parameterList.get(0).getEncodedParameter(), 0);
      int o2 = BitConverter.toInt(parameterList.get(1).getEncodedParameter(), 0);
      int res = o1 + o2;
      returnValue.setEncodedParameter(BitConverter.getBytes(res));
     }
     
     if(functionName.equals("xf_Sem_160103_external_functions_001")){
      int res = 1;
      returnValue.setEncodedParameter(BitConverter.getBytes(res));
     }
     
     if(functionName.equals("xf_Sem_160103_external_functions_002")){
         int o1 = BitConverter.toInt(parameterList.get(0).getEncodedParameter(), 0);
         int res = o1 + 1;
         returnValue.setEncodedParameter(BitConverter.getBytes(res));
      }
        
     //negative semantics cases always return 1
     if(functionName.equals("xf_NegSem_160103_external_functions_001")){
         int res = 1;
         returnValue.setEncodedParameter(BitConverter.getBytes(res));
      }

     if(functionName.equals("xf_NegSem_160103_external_functions_002")){
         int res = 1;
         returnValue.setEncodedParameter(BitConverter.getBytes(res));
      }

     if(functionName.equals("xf_NegSem_160103_external_functions_003")){
         int res = 1;
         returnValue.setEncodedParameter(BitConverter.getBytes(res));
      }

     if(functionName.equals("xf_NegSem_1602_toplevel_001")){
         int res = 1;
         returnValue.setEncodedParameter(BitConverter.getBytes(res));
      }

    
		return m_triOK;
	}

	public TriStatus triPAReset() {
		return m_triOK;
	}

	public TriStatus triReadTimer(TriTimerId timerId,
			TriTimerDuration elapsedTime) {
		return m_triOK;
	}

	public TriStatus triStartTimer(TriTimerId timerId,
			TriTimerDuration timerDuration) {
		return m_triOK;
	}
	
	public TriStatus triStopTimer(TriTimerId timerId) {
		return m_triOK;
	}
	
	public TriStatus triTimerRunning(TriTimerId timerId, TriBoolean running) {
		return m_triOK;
	}

	public void onConnectionClosed() {
		log("TRI connection closed");
		synchronized(this) { 
			notify();	
		}
	}

	public void onConnectionEstablished() {
		log("TRI connection established");
	}

	public void onError(String error) {
		log("TRI error occured: " + error);		
	}	
}
